@@ -219,7 +219,7 @@ test_that("numerically_balanced_group_factor_pairs_() unequal method on small da
     dplyr::group_by(case, condition) %>%
     dplyr::summarise(min_sd = min(sd_sum_)) %>%
     dplyr::left_join(aggregated_scores, by = c("case", "condition", "min_sd" = "sd_sum_")) %>%
-    dplyr::select(c(case, condition, min_sd, unequal_method, n_rows, n_folds)) %>%
+    dplyr::select(dplyr::all_of(c(case, condition, min_sd, unequal_method, n_rows, n_folds))) %>%
     dplyr::arrange(n_rows, case, condition, min_sd, unequal_method) %>%
     dplyr::group_by(case, condition) %>%
     dplyr::filter(dplyr::row_number() == 1)
@@ -443,6 +443,7 @@ test_that("experiment: numerically_balanced_group_factor_pairs_() optimizes for
   # dev.off()
 
   # 'sampling' is the random dataset
+  testthat::skip_if_not_installed("lmerTest")
   summary(lmerTest::lmer(diff_sum ~ opt_for + (1 | sampling), data = sim_wise_summaries))
   summary(lmerTest::lmer(diff_sd ~ opt_for + (1 | sampling), data = sim_wise_summaries))
 })
